//-----------------------------------------------------
// Klasse TicTacToe-Spiel (TTTSpiel) 
//-----------------------------------------------------
class TTTSpiel {

	// Attribute
	TTTSpieler spielerA, spielerB;

	TTTBrett brett;

	TTTRegeln regeln;

	// Konstruktor (Initialisierung eines Spiels)
	TTTSpiel(TTTSpieler a, TTTSpieler b, TTTBrett brett, TTTRegeln regeln) {
		this.spielerA = a;
		this.spielerB = b;
		this.brett = brett;
		this.regeln = regeln;
	}

	// Durchfuehrung eines Spiels
	void spielen() {
		this.brett.gebeSpielbrettAus();

		// Spieler A beginnt
		TTTSpieler aktSpieler = this.spielerA;
		TTTSpielzug zug = null;
		do { // abwechselndes Ziehen bis Spielende erreicht ist
			if (aktSpieler == this.spielerA)
				IO.println("Spieler A!");
			else
				IO.println("Spieler B ist am Zug!");
			// legalen Spielzug ermitteln und ausfuehren
			zug = aktSpieler.liefereNaechstenSpielzug(zug);
			if (!this.regeln.spielzugOK(zug)) {
				IO.println("Ungueltiger Spielzug! Verloren!");
				return;
			}
			this.brett.fuehreSpielzugAus(aktSpieler, zug);
			this.brett.gebeSpielbrettAus();
			// der andere Spieler ist nun ab Zug
			if (aktSpieler == spielerA)
				aktSpieler = this.spielerB;
			else
				aktSpieler = this.spielerA;
		} while (!this.regeln.spielEnde());
		// Spiel ist zuende
		this.regeln.gebeSiegerBekannt();
	}
}

// -----------------------------------------------------
// Klasse TicTacToe-Brett (TTTBrett)
// -----------------------------------------------------
class TTTBrett {

	// Konstanten
	final static int GROESSE = 3; // Brettgroesse

	final static TTTFigur LEER = null; // leeres Feld

	// Attribute
	TTTFigur[][] brett;

	// Konstruktor (Initialisierung des TTT-Brettes)
	TTTBrett() {
		// anfangs ist das Spielbrett (3x3-Matrix) leer
		this.brett = new TTTFigur[GROESSE][GROESSE];
		for (int i = 0; i < GROESSE; i++)
			for (int j = 0; j < GROESSE; j++)
				this.brett[i][j] = LEER;
	}

	// Ausfuehrung eines Spielzugs auf dem Brett
	void fuehreSpielzugAus(TTTSpieler spieler, TTTSpielzug zug) {
		// abhaengig vom aktuellen Spieler, wird eine
		// A_Figur oder eine B_Figur an die Stelle
		// gesetzt, die der Spielzug angibt
		int z = zug.getZeile();
		int s = zug.getSpalte();
		this.brett[z][s] = new TTTFigur(spieler.istSpielerA(), z, s);
	}

	// Ausgabe des aktuellen Spielbretts
	void gebeSpielbrettAus() {
		IO.print("  ");
		for (int j = 0; j < GROESSE; j++) {
			IO.print(j);
			IO.print(" ");
		}
		IO.println();

		IO.print("-");
		for (int j = 0; j < GROESSE; j++)
			IO.print("--");
		IO.println();

		for (int i = 0; i < GROESSE; i++) {
			IO.print(i);
			IO.print("|");
			for (int j = 0; j < GROESSE; j++) {
				if (this.brett[i][j] == LEER)
					IO.print(" ");
				else if (this.brett[i][j].istAFigur())
					IO.print(TTTFigur.getAFigur());
				else
					IO.print(TTTFigur.getBFigur());
				IO.print("|");
			}
			IO.println();

			IO.print("-");
			for (int j = 0; j < GROESSE; j++)
				IO.print("--");
			IO.println();
		}
	}

	// Lieferung der Figur an der Stelle z/s
	TTTFigur liefereFigur(int z, int s) {
		return this.brett[z][s];
	}

	// liefere die Brettgroesse
	static int getGroesse() {
		return GROESSE;
	}
}

// -----------------------------------------------------
// Klasse TicTacToe-Figur (TTTFigur)
// -----------------------------------------------------
class TTTFigur {

	// Konstanten
	final static char KREUZ = 'X';

	final static char KREIS = 'O';

	// Attribute
	boolean aOderB; // a == true

	int zeile;

	int spalte;

	// Konstruktor (Initialisierung einer Figur)
	TTTFigur(boolean aOderB, int zeile, int spalte) {
		this.aOderB = aOderB;
		this.zeile = zeile;
		this.spalte = spalte;
	}

	boolean istAFigur() {
		return this.aOderB;
	}

	boolean istBFigur() {
		return !this.istAFigur();
	}

	int getZeile() {
		return this.zeile;
	}

	int getSpalte() {
		return this.spalte;
	}

	static char getAFigur() {
		return KREUZ;
	}

	static char getBFigur() {
		return KREIS;
	}

}

// -----------------------------------------------------
// Klasse TicTacToe-Spielzug (TTTSpielzug)
// -----------------------------------------------------
class TTTSpielzug {

	// Attribute
	int zeile;

	int spalte;

	// Konstruktor (Initialisierung eines Spielzugs)
	TTTSpielzug(int z, int s) {
		this.zeile = z;
		this.spalte = s;
	}

	int getZeile() {
		return this.zeile;
	}

	int getSpalte() {
		return this.spalte;
	}
}

// -----------------------------------------------------
// Klasse TicTacToe-Regeln (TTTRegeln)
// -----------------------------------------------------
class TTTRegeln {

	// Konstanten
	final static boolean SPIELER_A = true;

	final static boolean SPIELER_B = !SPIELER_A;

	final static int SIEGER_A = 1;

	final static int SIEGER_B = 2;

	final static int UNENTSCHIEDEN = 0;

	final static int KEIN_SIEGER = -1;

	// Attribute
	TTTBrett brett;

	// Konstruktor (Initialisierung der Regeln)
	TTTRegeln(TTTBrett brett) {
		this.brett = brett;
	}

	// Ueberpruefung eines Spielzugs
	boolean spielzugOK(TTTSpielzug zug) {
		// ein Spielzug ist ok, wenn die Positionangaben
		// im gueltigen Bereich liegen und noch keine
		// Figur auf dem angegeben Feld existiert
		return (zug.getZeile() >= 0)
				&& (zug.getZeile() <= TTTBrett.getGroesse() - 1)
				&& (zug.getSpalte() >= 0)
				&& (zug.getSpalte() <= TTTBrett.getGroesse() - 1)
				&& (this.brett.liefereFigur(zug.getZeile(), zug.getSpalte()) == TTTBrett.LEER);
	}

	// Ueberpruefung des Spielendes
	boolean spielEnde() {
		// ein Spiel ist beendet, wenn alle Felder
		// besetzt sind oder wenn ein Sieger feststeht
		int besetzteFelder = 0;
		for (int i = 0; i < TTTBrett.getGroesse(); i++)
			for (int j = 0; j < TTTBrett.getGroesse(); j++)
				if (this.brett.liefereFigur(i, j) != null)
					besetzteFelder++;
		return (besetzteFelder == (TTTBrett.getGroesse() * TTTBrett
				.getGroesse()))
				|| (this.ermittleSieger() > UNENTSCHIEDEN);
	}

	// Ausgabe des Siegers
	void gebeSiegerBekannt() {
		int sieger = this.ermittleSieger();
		if (sieger == SIEGER_A)
			IO.println("Sieger ist Spieler A!");
		else if (sieger == SIEGER_B)
			IO.println("Sieger ist Spieler B!");
		else
			IO.println("Unentschieden!");
	}

	// Ermittlung des Siegers
	int ermittleSieger() {
		if (horizontale(SPIELER_A) || vertikale(SPIELER_A)
				|| diagonale(SPIELER_A)) {
			return SIEGER_A; // Sieger ist Spieler A
		}

		if (horizontale(SPIELER_B) || vertikale(SPIELER_B)
				|| diagonale(SPIELER_B)) {
			return SIEGER_B; // Sieger ist Spieler B
		}
		return KEIN_SIEGER; // noch kein Sieger
	}

	boolean vertikale(boolean istA) {
		for (int r = 0; r < TTTBrett.getGroesse(); r++) {
			int anzahl = 0;
			for (int s = 0; s < TTTBrett.getGroesse(); s++) {
				if (this.brett.liefereFigur(r, s) != TTTBrett.LEER
						&& this.brett.liefereFigur(r, s).istAFigur() == istA) {
					anzahl++;
				}
			}
			if (anzahl == TTTBrett.getGroesse()) {
				return true; // Sieger !
			}
		}
		return false;
	}

	boolean horizontale(boolean istA) {
		for (int s = 0; s < TTTBrett.getGroesse(); s++) {
			int anzahl = 0;
			for (int r = 0; r < TTTBrett.getGroesse(); r++) {
				if (this.brett.liefereFigur(r, s) != TTTBrett.LEER
						&& this.brett.liefereFigur(r, s).istAFigur() == istA) {
					anzahl++;
				}
			}
			if (anzahl == TTTBrett.getGroesse()) {
				return true; // Sieger !
			}
		}
		return false;
	}

	boolean diagonale(boolean istA) {
		// links nach rechts
		int anzahl = 0;
		for (int f = 0; f < TTTBrett.getGroesse(); f++) {
			if (this.brett.liefereFigur(f, f) != TTTBrett.LEER
					&& this.brett.liefereFigur(f, f).istAFigur() == istA) {
				anzahl++;
			}
		}
		if (anzahl == TTTBrett.getGroesse()) {
			return true; // Sieger !
		}

		// rechts nach links
		anzahl = 0;
		for (int f = 0; f < TTTBrett.getGroesse(); f++) {
			if (this.brett.liefereFigur(f, TTTBrett.getGroesse() - 1 - f) != TTTBrett.LEER
					&& this.brett
							.liefereFigur(f, TTTBrett.getGroesse() - 1 - f)
							.istAFigur() == istA) {
				anzahl++;
			}
		}
		if (anzahl == TTTBrett.getGroesse()) {
			return true; // Sieger !
		}
		return false;
	}
}
